# coding=utf-8
from .utils import parse, AcSource, AcDetail, resource_type_str_map, thin_string, limit_string

__author__ = 'dolacmeo'


class AcMoment(AcDetail):

    def __init__(self, acer, rid: [str, int]):
        if isinstance(rid, str) and rid.startswith('am'):
            rid = rid[2:]
        self.acer = acer
        super().__init__(acer, 10, rid)

    def loading_more(self):
        self.raw_data = self.raw_data.get("moment", {})

    @property
    def mobile_url(self):
        return f"{AcSource.routes['moment_mobile']}{self.resource_id}"

    @property
    def mobile_qrcode(self):
        parma = {
            "content": self.mobile_url,
            "contentType": "URL",
            "toShortUrl": False,
            "width": 100,
            "height": 100
        }
        return f"{AcSource.apis['qrcode']}?{parse.urlencode(parma)}"

    @property
    def title(self):
        if self.is_404:
            return self._msg['404']
        text = self.raw_data.get("replaceUbbText", "")
        return thin_string(text)

    @property
    def moment_data(self):
        return self.raw_data.get('moment')

    @property
    def source_data(self):
        return self.raw_data.get('repostSource')

    @property
    def source_obj(self):
        if not isinstance(self.source_data, dict):
            return None
        return self.acer.acfun.resource(self.source_data['resourceType'], self.source_data['resourceId'])

    @property
    def source_describe(self):
        if not isinstance(self.source_data, dict):
            return None
        rtype = resource_type_str_map[str(self.source_data['resourceType'])]
        rid = self.source_data['resourceId']
        content = limit_string(self.source_data.get("discoveryResourceFeedShowContent", ""), 10)
        content = ("#" + content) if len(content) else ""
        up_data = self.source_data['user']
        up_name = up_data['userName']
        return f"⋙{rtype}@{up_name}{content}"

    def __repr__(self):
        up_data = self.moment_data['user']
        up_name = up_data['name']
        txt_show = limit_string(self.title, 10)
        with_img = f"∮图x{len(self.moment_data['imgs'])}" if 'imgs' in self.moment_data else ""
        return f"AcMoment(@{up_name}#{txt_show}{with_img}{self.source_describe or ''})"


class MyMoment:
    cursor = "0"
    limit = 10
    moment_data = list()
    resourceTypes = 0
    rts = {  # 0综合 2视频 3文章 10动态
        'all': 0,
        'video': 2,
        'article': 3,
        'moment': 10
    }

    def __init__(self, acer):
        self.acer = acer

    @property
    def referer(self):
        rtypes = {
            "0": "?tab=all",
            "2": "?tab=video",
            "3": "?tab=article",
        }
        query = rtypes.get(str(self.resourceTypes), "")
        return f"{AcSource.routes['feeds']}{query}"

    def set_tab(self, tab: str = 'all') -> bool:
        new = self.rts.get(tab, 0)
        if new != self.resourceTypes:
            self.resourceTypes = new
            self.cursor = "0"
            self.moment_data = list()
            return True
        return False

    def feed(self, limit: int = 10, obj: bool = False, refresh: bool = False) -> (dict, None):
        if refresh is True:
            self.cursor = "0"
            self.moment_data = list()
        param = {
            "useWebp": False,
            "pcursor": self.cursor,
            "count": limit or self.limit,
            "resourceTypes": self.resourceTypes,
        }
        api_req = self.acer.client.get(AcSource.apis['follow_feed'], params=param)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.cursor = api_data.get('pcursor', self.cursor)
            self.moment_data.extend(api_data.get('feedList', []))
        if obj is False:
            return api_data.get('feedList', [])
        result = list()
        for x in api_data.get('feedList', []):
            result.append(self.acer.acfun.resource(x['resourceType'], x['resourceId']))
        return result



# coding=utf-8
from .utils import json, math, time, parse, Bs, Literal
from .utils import AcSource, AcDetail, not_404, match1

__author__ = 'dolacmeo'


class AcBangumi(AcDetail):

    def __init__(self, acer, rid: [str, int]):
        if isinstance(rid, str):
            if rid.startswith('aa'):
                rid = rid[2:]
            if "_36188_" in rid:
                rid, = map(int, rid.split('_36188_'))
        super().__init__(acer, 1, rid)

    @not_404
    def video(self, index: int = 0):
        assert index in range(len(self.episode_data))
        vid = self.episode_data[index]
        ends = f"_36188_{vid['videoId']}" if index > 0 else ""
        title = "" if len(self.episode_data) == 1 else vid['episodeName']
        return self.get_video(vid['videoId'], title, f"{self.referer}{ends}")

    @property
    @not_404
    def bangumi_data(self):
        return self.raw_data.get("data")

    @property
    @not_404
    def bangumi_list(self):
        return self.raw_data.get("list")

    @property
    @not_404
    def episode_data(self):
        return self.bangumi_list.get('items', [])

    @property
    def title(self):
        if self.is_404:
            return self._msg['404']
        return self.bangumi_data.get('bangumiTitle')

    @property
    def cover(self):
        if self.is_404:
            return None
        return self.cover_image()

    def cover_image(self, v_or_h: Literal["v", "h", "V", "H"] = "h"):
        return self.bangumi_data.get(f"bangumiCoverImage{v_or_h.upper()}")

    def __repr__(self):
        return f"AcBangumi([aa{self.resource_id}]{self.title})".encode(errors='replace').decode()

    @not_404
    def season(self, obj: bool = False):
        season_data = self.bangumi_data.get('relatedBangumis', [])
        if obj is False:
            return season_data
        seasons = list()
        for x in season_data:
            seasons.append(AcBangumi(self.acer, x['id']))
        return seasons

    def get_part_list(self, aa_num: [None, str] = None):
        if aa_num is None:
            return self.bangumi_list.get("items", [])
        param = {"quickViewId": "pagelet_partlist", "ajaxpipe": 1}
        api_req = self.acer.client.get(f"{AcSource.routes['bangumi']}{aa_num}", params=param)
        assert api_req.text.endswith("/*<!-- fetch-stream -->*/")
        api_data = json.loads(api_req.text[:-25])
        v_script = match1(api_data['html'].replace(" ", ""), r"partsData=((?=\{)[^\s]*(?<=\}))\<\/script\>")
        part_list = json.loads(v_script)
        return part_list.get("items", [])


class AcBangumiList:
    page_obj = None
    menu_filter = list()
    default_filter = None

    def __init__(self, acer):
        self.acer = acer
        self.loading()

    def __repr__(self):
        return f"AcBangumiList(番剧列表 - AcFun弹幕视频网)"

    @property
    def referer(self):
        return f"{AcSource.routes['bangumi_list']}"

    def loading(self):
        index_req = self.acer.client.get(AcSource.routes['bangumi_list'])
        self.page_obj = Bs(index_req.text, 'lxml')
        firsts = []
        for menu in self.page_obj.select(".ac-menu .ac-menu-filter"):
            menu_id = int(menu.attrs['data-id'])
            filter_title = menu.select_one('.ac-menu-filter-title').text.replace(":", "")
            filter_item = {item.attrs['data-id']: item.text for item in menu.select(".ac-menu-filter-item")}
            self.menu_filter.append({'title': filter_title, 'item': filter_item, 'menu_id': menu_id})
            default_id = menu.select_one(".ac-menu-filter-item").attrs['data-id']
            firsts.append([menu_id, default_id])
        self.menu_filter = sorted(self.menu_filter, key=lambda m: m['menu_id'])
        firsts = sorted(firsts, key=lambda i: i[0])
        self.default_filter = ",".join([n[1] for n in firsts])

    def _filter_check(self, filters: [str, list]):
        filters = filters.split(",") if isinstance(filters, str) else filters
        if len(filters) != len(self.menu_filter):
            return False
        for n in range(len(filters)):
            if filters[n] not in self.menu_filter[n]['item']:
                print(f"{filters[n]} not in {self.menu_filter[n]['title']}: {self.menu_filter[n]['item'].keys()}")
                return False
        return True

    def tans_filters(self, words: [str, list]) -> (str, bool):
        filters = words.split(",") if isinstance(words, str) else words
        if len(filters) != len(self.menu_filter):
            return False
        v = list()
        for n in range(len(filters)):
            if filters[n] not in self.menu_filter[n]['item'].values():
                return False
            _inverse = {v: k for k, v in self.menu_filter[n]['item'].items()}
            v.append(_inverse[filters[n]])
        return ",".join(v)

    def page_from_url(self, full_url: str, obj: bool = False):
        filters = self.default_filter
        page = 1
        param = parse.urlsplit(full_url).query
        param = parse.parse_qs(param)
        if "filters" in param:
            filters = param['filters'][0]
        if "pageNum" in param:
            page = int(param['pageNum'][0])
        return self.page(filters, page, obj)

    def page(self, filters: [str, list, None] = None, page: int = 1, obj: bool = False) -> (dict, None):
        filters = self.default_filter if filters is None else filters
        if self._filter_check(filters) is False:
            return None
        filters = ",".join(filters) if isinstance(filters, list) else filters
        param = {
            "quickViewId": "bangumiList",
            "filters": filters,
            "reqID": 1, "ajaxpipe": 1,
            "pageNum": page,
            "t": str(time.time_ns())[:13]
        }
        page_req = self.acer.client.get(AcSource.routes['bangumi_list'], params=param)
        if not page_req.text.endswith("/*<!-- fetch-stream -->*/"):
            return None
        api_data = json.loads(page_req.text[:-25])
        page_obj = Bs(api_data.get('html', ''), 'lxml')
        ac_list = page_obj.select_one(".ac-mod-list")
        item_total = int(ac_list.attrs['data-totalcount'])
        page_size = int(ac_list.attrs['data-pagesize'])
        max_page = math.ceil(item_total / page_size)
        items = list()
        for item in ac_list.select("li.ac-mod-li"):
            item_data = {
                "aa_num": item.attrs['data-aid'],
                "link": item.select_one('.ac-mod-link').attrs['href'],
                "cover": item.select_one('img.ac-mod-cover').attrs['src'],
                "cover_stats": item.select_one('.ac-mod-cover-stats').text,
                "need_pay": item.select_one('.pay-toast') is not None,
                "title": item.select_one('.ac-mod-title').text,
                "latestItem": item.select_one('.ac-mod-title > em').text,
            }
            if obj is True:
                items.append(AcBangumi(self.acer, item_data['aa_num']))
            else:
                items.append(item_data)
        return {
            "items": items,
            "page_now": page,
            "page_total": max_page,
            "page_size": page_size,
            "total_count": item_total
        }

    pass



# coding=utf-8
import os
import re
import json
import time
import math
import httpx
import base64
import subprocess
from uuid import uuid4
from urllib import parse
from typing import Literal
from datetime import timedelta
from bs4 import BeautifulSoup as Bs
from bs4.element import Tag
from ..source import AcSource
from ..exceptions import need_login, not_404

__author__ = 'dolacmeo'

resource_type_map = {
    "1": "AcBangumi",  # 番剧
    "2": "AcVideo",  # 视频稿件
    "3": "AcArticle",  # 文章稿件
    "4": "AcAlbum",  # 合辑
    "5": "AcUp",  # 用户
    "6": "AcComment",  # 评论
    # "8": "私信",
    "10": "AcMoment",  # 动态
}

routes_type_map = {
    "bangumi": "AcBangumi",  # 番剧
    "video": "AcVideo",  # 视频稿件
    "article": "AcArticle",  # 文章稿件
    "album": "AcAlbum",  # 合辑
    "up": "AcUp",  # 用户
    "live": "AcLiveUp",  # 用户直播
    "moment": "AcMoment",  # 用户动态
    "doodle": "AcDoodle",  # 涂鸦页面
}

type_routes_map = {v: k for k, v in routes_type_map.items()}

resource_type_str_map = {
    "1": "番剧",  # AcBangumi
    "2": "视频",  # AcVideo
    "3": "文章",  # AcArticle
    "4": "合辑",  # AcAlbum
    "5": "用户",  # AcUp
    "6": "评论",  # AcComment
    "10": "动态",  # AcMoment
}

acpage_regex = {
    "AcBangumi": [
        r"(?s)bangumiData\s*=\s*(\{.*?\});",
        r"(?s)bangumiList\s*=\s*(\{.*?\});"
    ],  # 番剧
    "AcVideo": [r"(?s)videoInfo\s*=\s*(\{.*?\});"],  # 视频稿件
    "AcArticle": [r"(?s)articleInfo\s*=\s*(\{.*?\});"],  # 文章稿件
    "AcAlbum": [r"(?s)__INITIAL_STATE__\s*=\s*(\{.*?\});"],  # 合辑
    "AcMoment": [r"(?s)__INITIAL_STATE__\s*=\s*(\{.*?\});"],  # 动态
    "AcDoodle": [r"(?s)__schema__\s*=\s*'(\{.*?\})';"],  # 涂鸦页面
    "AcUp": [],  # 用户
}


class VideoItem:
    raw_data = None

    def __init__(self, acer, video_id: int, sub_title: str, referer: str, parent):
        self.acer = acer
        self.vid = video_id
        self.sub_title = sub_title
        self.referer = referer
        self.parent = parent
        self.loading()

    def __repr__(self):
        return f"{self.parent}"

    def _get_data_from_quick_view(self) -> (dict, None):
        param = {"quickViewId": "videoInfo_new", "ajaxpipe": 1}
        page_req = self.acer.client.get(self.referer, params=param)
        assert page_req.text.endswith("/*<!-- fetch-stream -->*/")
        page_data = json.loads(page_req.text[:-25])
        v_script = match1(page_data['html'].replace(" ", ""), r"videoInfo=((?=\{)[^\s]*(?<=\}));?")
        video_data = json.loads(v_script)
        return video_data.get("currentVideoInfo")

    def _get_data_from_api(self) -> (dict, None):
        param = {
            "resourceId": self.parent.resource_id,
            "resourceType": self.parent.resource_type,
            "videoId": self.vid
        }
        api_req = self.acer.client.get(AcSource.apis['video_ksplay'], params=param)
        api_data = api_req.json()
        assert api_data.get('result') == 0
        return api_data.get("playInfo")

    @property
    def quality(self) -> list:
        return self.raw_data.get("transcodeInfos", [])

    def m3u8_url(self, quality: [int, str] = 0, hevc: bool = True, only_url: bool = True) -> (dict, str, None):
        if isinstance(quality, int):
            assert quality in range(len(self.quality))
        elif isinstance(quality, str):
            quality = quality.lower()
            q_map = {x["qualityType"]: i for i, x in enumerate(self.quality)}
            assert quality in q_map.keys()
            quality = q_map[quality]
        else:
            return None
        code_type = "ksPlayJsonHevc" if hevc is True else "ksPlayJson"
        play_data = json.loads(self.raw_data.get(code_type, ""))
        this_quality = play_data.get("adaptationSet", [{}])[0].get("representation")[quality]
        domains = self.raw_data.get("domainInfos", [])
        if len(domains):
            main_url = parse.urlsplit(this_quality['url'])
            this_quality['backupUrl'] = list()
            for domain in domains:
                new_url = f"{main_url.scheme}://{domain['url']}{main_url.path}?{main_url.query}"
                this_quality['backupUrl'].append(new_url)
        if only_url is True:
            return this_quality['url'], this_quality['backupUrl']
        return this_quality

    def play(self, potplayer_path: [os.PathLike, str], quality: [int, str] = "1080p", hevc: bool = True):
        assert os.path.exists(potplayer_path)
        adapt = self.m3u8_url(quality, hevc, False)
        qtype = adapt["qualityType"]
        quality_mark = f"{qtype}_HEVC" if hevc is True else qtype
        title = f"-{self.sub_title}" if len(self.sub_title) else ""
        player_title = f'"{self.parent}{title}-{quality_mark}"'.replace(" ", '')
        cmds = [potplayer_path, adapt['url'], "/title", emoji_cleanup(player_title)]
        return subprocess.Popen(cmds, stdout=subprocess.PIPE)

    @property
    def scenes(self) -> (dict, None):
        form_data = {
            "resourceId": self.parent.resource_id,
            "resourceType": self.parent.resource_type,
            "videoId": self.vid
        }
        api_req = self.acer.client.post(AcSource.apis['video_scenes'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        if api_data.get("spriteVtt") is None:
            return None
        pos_data = list()
        sprite_data = api_data.get("spriteVtt", "").split("\n\n")[1:]
        sprite_img = sprite_data[0].split("\n")[1].split("#")[0]
        for line in sprite_data:
            pos, img_url = line.split("\n")
            pos_s, pos_e = pos.split(" --> ")
            _, xywh = img_url.split("#xywh=")
            pos_data.append([pos_s, pos_e, xywh])
        return {"sprite_image": sprite_img, "pos": pos_data}

    @property
    def hotspot(self) -> (dict, None):
        form_data = {
            "resourceId": self.parent.resource_id,
            "resourceType": self.parent.resource_type
        }
        api_req = self.acer.client.post(AcSource.apis['video_hotspot'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') != 0:
            return None
        return api_data.get("hotSpotDistribution")

    def loading(self):
        self.raw_data = self._get_data_from_api()

    @property
    def danmaku(self) -> object:
        return self.acer.acfun.AcDanmaku(self.vid, self.parent)


class AcDetail:
    resource_type = None
    resource_id = None
    is_404 = False
    raw_data = dict()
    page_text = None
    page_obj = None
    pagelets = list()
    __funcs = {
        "AcBangumi": ['get_video', 'comment', 'like', 'favorite', 'banana'],
        "AcVideo":   ['up', 'up_uid', 'up_name', 'get_video', 'comment', 'like', 'favorite', 'banana'],
        "AcArticle": ['up', 'up_uid', 'up_name', 'comment', 'like', 'favorite', 'banana'],
        "AcAlbum":   ['favorite'],
        "AcMoment":  ['up', 'up_uid', 'up_name', 'comment', 'like', 'banana'],
        "AcDoodle":  ['comment']
    }
    _msg = {
        "404": "咦？世界线变动了。看看其他内容吧~"
    }

    def __init__(self, acer, rtype: [str, int], rid: [str, int]):
        self.acer = acer
        self.resource_type = int(rtype)
        self.resource_id = int(rid)
        self.loading()

    @property
    def _objname(self):
        return self.__class__.__name__

    @property
    def referer(self):
        route_name = type_routes_map[self._objname]
        return f"{AcSource.routes[route_name]}{self.resource_id}"

    @property
    def qrcode(self):
        parma = {
            "content": self.referer,
            "contentType": "URL",
            "toShortUrl": False,
            "width": 100,
            "height": 100
        }
        return f"{AcSource.apis['qrcode']}?{parse.urlencode(parma)}"

    @property
    def title(self):
        return ""

    def __repr__(self):
        if self.is_404:
            return f"{self._objname}([#{self.resource_id}]咦？世界线变动了。看看其他内容吧~)"
        return f"{self._objname}([#{self.resource_id}]{self.title})"

    def loading(self):
        req = self.acer.client.get(self.referer)
        self.is_404 = req.status_code // 100 != 2
        if self.is_404:
            return False
        self.page_text = req.text
        self.page_obj = Bs(req.text, 'lxml')
        self.pagelets = get_page_pagelets(self.page_obj)
        rex = acpage_regex.get(self._objname, [])
        if self._objname == "AcBangumi":
            script_bangumidata, script_bangumilist = match1(req.text, *rex)
            self.raw_data = dict(data=json.loads(script_bangumidata), list=json.loads(script_bangumilist))
        elif len(rex):
            self.raw_data = json.loads(match1(req.text, *rex))
        if "ssrContext" in self.raw_data.keys():
            route_name = type_routes_map[self._objname]
            self.raw_data = self.raw_data.get(route_name)
        if self._objname in self.__funcs.keys():
            for k in self.__funcs[self._objname]:
                self.__setattr__(k, getattr(self, f"_{k}"))
        self.loading_more()

    def loading_more(self):
        pass

    @property
    @not_404
    def _up_uid(self):
        user = self.raw_data.get('user', {})
        return user.get("id")

    @property
    @not_404
    def _up_name(self):
        user = self.raw_data.get('user', {})
        return user.get("name", "")

    @not_404
    def _up(self):
        return self.acer.acfun.AcUp(self._up_uid)

    @not_404
    def _get_video(self, video_id: int, sub_title: str, referer: str):
        return VideoItem(self.acer, video_id, sub_title, referer, self)

    @not_404
    def _comment(self) -> object:
        return self.acer.acfun.AcComment(self.resource_type, self.resource_id)

    @not_404
    def _like(self, on_off: bool):
        if on_off is True:
            return self.acer.like_add(self.resource_type, self.resource_id)
        return self.acer.like_delete(self.resource_type, self.resource_id)

    @not_404
    def _favorite(self, on_off: bool):
        if on_off is True:
            return self.acer.favourite.add(self.resource_type, self.resource_id)
        return self.acer.favourite.cancel(self.resource_type, self.resource_id)

    @not_404
    def _banana(self, count: int):
        assert 1 >= count >= 5
        return self.acer.throw_banana(self.resource_type, self.resource_id, count)

    @not_404
    def report(self, crime: str, proof: str, description: str):
        return self.acer.acfun.AcReport.submit(
            self.referer, self.resource_type, self.resource_id, self._up_uid,
            crime, proof, description)


class B64s:
    STANDARD = b'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/'
    EN_TRANS = STANDARD
    DE_TRANS = STANDARD

    def __init__(self, s: [bytes, bytearray], n: [int, None] = None):
        self.raw = s
        if isinstance(n, int):
            n = n % 64
            new = self.STANDARD[n:] + self.STANDARD[:n]
            self.EN_TRANS = bytes.maketrans(self.STANDARD, new)
            self.DE_TRANS = bytes.maketrans(new, self.STANDARD)

    def b64encode(self):
        return base64.standard_b64encode(self.raw).translate(self.EN_TRANS)

    def b64decode(self):
        return base64.b64decode(self.raw.translate(self.DE_TRANS))


def sizeof_fmt(num, suffix="B") -> str:
    for unit in ["", "Ki", "Mi", "Gi", "Ti", "Pi", "Ei", "Zi"]:
        if abs(num) < 1024.0:
            return f"{num:3.1f} {unit}{suffix}"
        num /= 1024.0
    return f"{num:.1f} Yi{suffix}"


def match_info(text) -> (dict, None):
    rex = re.compile(r"(?P<title>.*)\r"
                     r"UP:(?P<up>.*)\r"
                     r"发布于(?P<createTime>\d{4}-\d{2}-\d{2} \d{2}:\d{2}:\d{2})"
                     r"\xa0/\xa0点击:(?P<viewCount>\d+)"
                     r"\xa0/\xa0评论:(?P<commentCount>\d+)")
    result = rex.fullmatch(text)
    if result is None:
        return None
    return result.groupdict()


def match1(text, *patterns):
    """Scans through a string for substrings matched some patterns (first-subgroups only).

    Args:
        text: A string to be scanned.
        patterns: Arbitrary number of regex patterns.

    Returns:
        When only one pattern is given, returns a string (None if no match found).
        When more than one pattern are given, returns a list of strings ([] if no match found).
    """

    if len(patterns) == 1:
        pattern = patterns[0]
        match = re.search(pattern, text)
        if match:
            return match.group(1)
        else:
            return None
    else:
        ret = []
        for pattern in patterns:
            match = re.search(pattern, text)
            if match:
                ret.append(match.group(1))
        return ret


def matchall(text, patterns):
    """Scans through a string for substrings matched some patterns.

    Args:
        text: A string to be scanned.
        patterns: a list of regex pattern.

    Returns:
        a list if matched. empty if not.
    """

    ret = []
    for pattern in patterns:
        match = re.findall(pattern, text)
        ret += match

    return ret


def emoji_cleanup(text) -> str:
    # Ref: https://gist.github.com/Alex-Just/e86110836f3f93fe7932290526529cd1#gistcomment-3208085
    # Ref: https://en.wikipedia.org/wiki/Unicode_block
    EMOJI_PATTERN = re.compile(
        "(["
        "\U0001F1E0-\U0001F1FF"  # flags (iOS)
        "\U0001F300-\U0001F5FF"  # symbols & pictographs
        "\U0001F600-\U0001F64F"  # emoticons
        "\U0001F680-\U0001F6FF"  # transport & map symbols
        "\U0001F700-\U0001F77F"  # alchemical symbols
        "\U0001F780-\U0001F7FF"  # Geometric Shapes Extended
        "\U0001F800-\U0001F8FF"  # Supplemental Arrows-C
        "\U0001F900-\U0001F9FF"  # Supplemental Symbols and Pictographs
        "\U0001FA00-\U0001FA6F"  # Chess Symbols
        "\U0001FA70-\U0001FAFF"  # Symbols and Pictographs Extended-A
        "\U00002702-\U000027B0"  # Dingbats
        "])"
    )
    text = re.sub(EMOJI_PATTERN, r'', text)
    return text


def image_uploader(client, image_data: bytes, ext: str = 'jpeg') -> str:
    token_req = client.post(AcSource.apis['image_upload_gettoken'], data=dict(fileName=uuid4().hex.upper() + f'.{ext}'))
    token_data = token_req.json()
    assert token_data.get('result') == 0
    resume_req = client.get(AcSource.apis['image_upload_resume'], params=dict(upload_token=token_data['info']['token']))
    resume_data = resume_req.json()
    assert resume_data.get('result') == 1
    fragment_req = client.post(AcSource.apis['image_upload_fragment'], data=image_data,
                               params=dict(upload_token=token_data['info']['token'], fragment_id=0),
                               headers={"Content-Type": "application/octet-stream"})
    fragment_data = fragment_req.json()
    assert fragment_data.get('result') == 1
    complete_req = client.post(AcSource.apis['image_upload_complete'],
                               params=dict(upload_token=token_data['info']['token'], fragment_count=1))
    complete_data = complete_req.json()
    assert complete_data.get('result') == 1
    result_req = client.post(AcSource.apis['image_upload_geturl'], data=dict(token=token_data['info']['token']))
    result_data = result_req.json()
    assert result_data.get('result') == 0
    return result_data.get('url')


def limit_string(s, n) -> str:
    s = s[:n]
    if len(s) == n:
        s = s[:-2] + "..."
    return s


def thin_string(_string: str, no_break: bool = False) -> str:
    final_str = list()
    for line in _string.replace('\r', '').split('\n'):
        new_line = ' '.join(line.split()).strip()
        if len(new_line):
            final_str.append(new_line)
    if no_break is True:
        return " ".join(final_str)
    return " ↲ ".join(final_str)


def warp_mix_chars(_string: str, lens: int = 40, border: [tuple, None] = None) -> (str, list):
    output = list()
    tmp_string = ""
    tmp_count = 0
    for i, _char in enumerate(_string):
        char_count = 2 if '\u4e00' <= _char <= '\u9fcc' else 1
        if tmp_count + char_count > lens:
            tmp_string += ' ' * (lens - tmp_count)
            output.append(tmp_string)
            tmp_string = ""
            tmp_count = 0
        tmp_string += _char
        tmp_count += char_count
    if tmp_count != 0:
        tmp_string += ' ' * (lens - tmp_count)
        output.append(tmp_string)
    if isinstance(border, tuple) and len(border) == 2:
        output = [f"{border[0]}{x}{border[1]}" for x in output]
    return output


def ms2time(ms: int) -> str:
    d = timedelta(milliseconds=ms)
    return str(d).split('.')[0]


def get_page_pagelets(page_obj) -> list:
    data = list()
    for item in page_obj.select("[id^=pagelet_]"):
        data.append(item.attrs['id'])
    if len(data):
        data.append('footer')
    return data


def url_complete(url) -> str:
    if isinstance(url, str):
        if url.startswith('//'):
            url = f"https:{url}"
        elif not url.startswith('http'):
            url = f"{AcSource.routes['index']}{url}"
    return url


def get_dict_value_in_path(data: dict, path: [str, list], default=None):
    if isinstance(path, str):
        path = path.split('.')
    inside = data
    for i, x in enumerate(path):
        v = default if i == len(path) - 1 else {}
        inside = inside.get(x, v)
    return inside



# coding=utf-8
from .utils import AcSource, AcDetail, not_404

__author__ = 'dolacmeo'


class AcArticle(AcDetail):

    def __init__(self, acer, rid: [str, int]):
        if isinstance(rid, str) and rid.startswith('ac'):
            rid = rid[2:]
        super().__init__(acer, 3, rid)

    @property
    def title(self):
        if self.is_404:
            return self._msg['404']
        return self.raw_data.get('contentTitle', self.raw_data.get('title', ""))

    @property
    def tags(self):
        if self.is_404:
            return self._msg['404']
        return self.raw_data.get('tagList', [])

    @property
    def contents(self):
        if self.is_404:
            return self._msg['404']
        return self.raw_data.get('parts', [])

    @property
    def cover(self):
        if self.is_404:
            return None
        return self.raw_data.get("coverUrl")

    def __repr__(self):
        user_name = self._up_name or self._up_uid
        user_txt = "" if len(user_name) == 0 else f" @{user_name}"
        return f"AcArticle([ac{self.resource_id}]{self.title}{user_txt})".encode(errors='replace').decode()

    @not_404
    def recommends(self, obj: bool = False) -> (dict, None):
        articles = list()
        for item in self.page_obj.select('#main > section.area > .content > .fr > .contribution.weblog-item'):
            this_url = item.select_one('.contb-title a').attrs['href']
            ac_num = this_url[5:]
            data = {
                'title': item.select_one('.contb-title').a.text,
                'shareUrl': AcSource.routes['share'] + ac_num,
                'viewCount': item.select_one('.contb-count .view-count span.count').text,
                'commentCount': item.select_one('.contb-count .comm-count span.count').text,
                'user': self.raw_data.get('user', {})
            }
            if obj is True:
                articles.append(AcArticle(self.acer, ac_num))
            else:
                articles.append(data)
        return articles

    def AcWen(self) -> (object, None):
        if self.is_404:
            return None
        rid = self.raw_data.get("realm", {}).get("realmId")
        return AcWen(self.acer, [rid])

    def AcChannel(self) -> (object, None):
        if self.is_404:
            return None
        cid = self.raw_data.get("channel", {}).get('id')
        return self.acer.acfun.AcChannel(cid)


class AcWen:
    realmIds = None
    cursor = "first_page"
    sortType = "createTime"
    onlyOriginal = False
    timeRange = "all"
    limit = 10
    article_data = list()

    def __init__(self, acer,
                 realmIds: [list, None] = None,
                 sortType: str = "createTime",
                 timeRange: str = "all",
                 onlyOriginal: bool = False,
                 limit: int = 10):
        self.acer = acer
        self.realmIds = realmIds
        assert sortType in ['createTime', 'lastCommentTime', 'hotScore']
        self.sortType = sortType
        assert timeRange in ['all', 'oneDay', 'threeDay', 'oneWeek', 'oneMonth']
        self.timeRange = timeRange
        self.onlyOriginal = onlyOriginal
        self.limit = limit

    @property
    def referer(self):
        return f"{AcSource.routes['index']}/v/list63/index.htm"

    def feed(self, obj: bool = True) -> (dict, None):
        if self.cursor == 'no_more':
            return None
        form_data = {
            "cursor": self.cursor,
            "sortType": self.sortType,
            "onlyOriginal": self.onlyOriginal,
            "timeRange": self.timeRange,
            "limit": self.limit,
        }
        if isinstance(self.realmIds, list):
            form_data['realmId'] = self.realmIds
        api_req = self.acer.client.post(AcSource.apis['article_feed'], data=form_data)
        api_data = api_req.json()
        if api_data.get('result') == 0:
            self.cursor = api_data.get('cursor')
            new_data = api_data.get('data', [])
            self.article_data.extend(new_data)
            if obj is True:
                return [AcArticle(self.acer, x['articleId'], x) for x in new_data]
            return new_data
        return None

    def clean_cache(self) -> bool:
        self.article_data = list()
        return True
